#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé Mars 2024
@auteur: David ALBERTO (www.astrolabe-science.fr)

QUELQUES FONCTIONS UTILES EN GNOMONIQUE.
"""
from math import sin, cos, tan, asin, acos, atan2, radians, degrees, pi
import datetime as dt

eps = radians(23.439)# obliquité de l'écliptique


#  -------------------------- FONCTIONS

def rang(jour, mois):
    """
    Parameters
    ----------
    jour, mois : TYPE : deux entiers
        DESCRIPTION : n° du jour dans le mois, n° du mois dans l'année
    Returns
    -------
    Renvoie le rang du jour dans l'année (entier).
    Année non bissextile.
    """
    date = dt.date(2026, mois, jour)
    unjanvier = dt.date(2026,1,1)
    J = (date - unjanvier).days + 1
    return J

def declinaison(J, deg = False):
    """
    Parameters
    ----------
    J : TYPE : entier
        DESCRIPTION : rang du jour dans l'année.
    deg : booléen (True si on veut la valeur en degrés)
    Returns
    -------
    Déclinaison du Soleil (radians par défaut, degrés si deg=True).
    Cette modélisation ne donne qu'une indication.
    """
    long = (2*pi*(J-81))/365.2422
    decl = asin(sin(eps)*sin(long))
    if deg:decl = degrees(decl)
    return decl # en radians

def declinaison_2(jour, mois, annee, deg = False):
    """
    D'après D. Savoie (Gnomonique).
    Parameters
    ----------
    jour, mois, annee : entiers. Numéros correspondant à la date souhaitée.
    deg : booléen (True si on veut la valeur en degrés)
    Returns
    -------
    Renvoie la déclinaison du Soleil à la date demandée (radians par défaut, 
                                                         en degrés si deg=True).
    Ce calcul est plus précis que celui de la fonction declinaison().
    """
    Tzero = dt.datetime(2000, 1, 1, 12) #  1er janvier 2000 12hTU, date de référence
    date = dt.datetime(annee, mois, jour)
    T = (date - Tzero).days
    T = T + (date - Tzero).seconds / 3600 / 24
    LM = (280.460 + 0.9856474 * T) % 360
    M = (357.528 + 0.9856003 * T) % 360
    long = (LM + 1.915 * sin(radians(M)) + 0.020 * sin(radians(2*M))) % 360
    # AD = arctan(cos(eps) * tan(radians(long)))
    declinaison = asin(sin(eps) * sin(radians(long)))
    if deg:
        declinaison = degrees(declinaison)
    return declinaison

def arc_semidiurne(latitude, rang, deg = False):
    """
    latitude : TYPE float
        DESCRIPTION. latitude d'un lieu, en degrés.
    rang : TYPE : entier
        DESCRIPTION. rang du jour dans l'année (de 1 à 365)
    deg : booléen (True si on veut la valeur en degrés)
    Returns
    -------
    L'angle horaire au lever/au coucher du Soleil
    (en radians par défaut, degrés si deg=True).
    """
    decl = declinaison(rang)
    lat_rad = radians(latitude)
    H0 = acos(- tan(lat_rad) * tan(decl))
    if deg:
        H0 = degrees(H0)
    return H0

def azimut(latitude, J, H, deg = False):
    """
    latitude : float
    latitude du lieu (radians).
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians)
    deg : booléen (True si on veut la valeur en degrés)
    -----------------
    Renvoie l'azimut du Soleil compté depuis le sud (radians par défaut,
    en degrés si deg=True)
    """
    decl = declinaison(J)
    A = atan2(sin(H), (sin(latitude) * cos(H) - cos(latitude) * tan(decl)))
    if deg:
        A = degrees(A)
    return A

def hauteur(latitude, J, H, deg = False):
    """
    latitude : float
    latitude du lieu (radians).
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians)
    deg : booléen (True si on veut la valeur en degrés)
    ------------------
    Renvoie la hauteur du Soleil en radians, par défaut, degrés si deg = True.
    """
    decl = declinaison(J)
    sinh = sin(latitude)*sin(decl) + cos(latitude)*cos(decl)*cos(H)
    h = asin(sinh)
    if deg:
        h = degrees(h)
    return h

def heure_AH(H):
    """
    Passe de l'angle horaire du Soleil à l'heure en format décimal.
    Parameters
    ----------
    H : TYPE : float
        DESCRIPTION : angle horaire (degrés)
    Returns
    -------
    heure en format décimal.
    """
    heure = 12 + H/15
    return heure

def AH_passage_mur(declinaison, D, latitude):
    """
    J (entier) : rang du jour dans l'année (1 à 365)
    declinaison : (float) déclinaison du Soleil (degrés)
    D (float) : déclinaison gnomonique du mur (degrés)
    latitude (float) : latitude du lieu (degres).
    -------------
    Renvoie les deux angles horaires de passage du Soleil dans le plan 
    d'un mur vertical, pendant la journée, en degres (tuple).
    """
    decl = radians(declinaison)
    D_rad = radians(D)
    lat_r = radians(latitude)
    S = atan2(tan(D_rad) / sin(lat_r),1)
    t = acos(cos(S) * tan(decl) * cos(lat_r) / sin(lat_r))
    Hmin = S - t
    Hmax = S + t
    return degrees(Hmin), degrees(Hmax)

def lever_coucher(J, latitude):
    """
    Parameters
    ----------
    J : entier
        DESCRIPTION : rang du jour dans l'année (1 à 365)
    latitude : float
        DESCRIPTION : latitude du lieu (degres).
    Returns
    -------
    Renvoie les heures de lever et de coucher du Soleil (tuple) en format décimal.
    """
    H0 = arc_semidiurne(latitude, J, deg = True)
    Hlever = 12 - H0/15
    Hcoucher = 12 + H0/15
    return Hlever, Hcoucher

def HHmm(H):
    """
    Transforme l'heure décimale en format sexagésimal.
    Parameters
    ----------
    H : TYPE : float
        DESCRIPTION : heure en format décimal.
    -------
    Returns : heure en format string (heure:min)
    """
    heures = int(H)
    minutes = round((H-heures)*60)
    if minutes == 60:
        minutes = '00'
        heures = heures + 1
    elif minutes <10:
        minutes = '0' + str(minutes)
    heure_60 = f"{heures}h{minutes}min"
    return heure_60

def HHmmss(H):
    """
    Transforme l'heure décimale en format sexagésimal.
    Parameters
    ----------
    H : TYPE : float
        DESCRIPTION : heure en format décimal.
    -------
    Returns : heure en format string (heure:min:seconde)
    """
    heures = int(H)
    minutes = int((H-heures)*60)
    secondes = round((((H-heures) * 60 - minutes)) * 60)
    if minutes <10:
        minutes = '0' + str(minutes)
    if secondes <10:
        secondes = '0' + str(secondes)
    heure_60 = f"{heures}h{minutes}min{secondes}s"
    return heure_60

def heure_passage_mur(declinaison, D, latitude):
    """
    Parameters
    ----------
    declinaison : (float) : déclinaison du Soleil (degrés)
    D : (float) : déclinaison gnomonique du mur (degrés)
    latitude (float) : latitude (degrés)
    Returns
    -------
    heures de passage du Soleil dans le plan du mur (string, tuple), format HHmmss
    """
    matin = HHmmss(heure_AH(AH_passage_mur(declinaison, D, latitude)[0]))
    soir = HHmmss(heure_AH(AH_passage_mur(declinaison, D, latitude)[1]))
    return matin, soir


def equation_temps(jour, mois, annee):
    """
    Calcule l'équation du temps pour une date donnée.
    D'après D. Savoie (Gnomonique) Précision env.30 s pour 1950-2050.
    Parameters
    ----------
    date : TYPE : date au format datetime : dt.datetime(annee, mois , jour)
        DESCRIPTION : date du jour souhaité.
    Returns
    -------
    Renvoie la valeur de l'équation du temps à la date demandée,
    sous forme de tuple : (valeur décimale, string en format min:sec).
    """
    Tzero = dt.datetime(2000, 1, 1, 12) #  1er janvier 2000 12hTU, date de référence
    date = dt.datetime(annee, mois, jour)
    T = (date - Tzero).days
    T = T + (date - Tzero).seconds / 3600 / 24
    LM = (280.460 + 0.9856474 * T) % 360
    M = (357.528 + 0.9856003 * T) % 360
    long = (LM + 1.915 * sin(radians(M)) + 0.020 * sin(radians(2*M))) % 360
    AD = atan2(cos(eps) * sin(radians(long)), cos(radians(long)))
    EdT_degres = ((degrees(AD) - LM))%360
    if EdT_degres > 180:
        EdT_degres = EdT_degres - 360
    EdT = EdT_degres * 4
    minutes = int(EdT)
    secondes = round((abs(EdT - minutes)) * 60,1)
    return EdT, f"{minutes}min{secondes}s"

def longitude_date(jour, mois, annee, deg = False) :
    """
    D'après D. Savoie (Gnomonique).
    Parameters
    ----------
    date : TYPE : date au format datetime
        DESCRIPTION : date du jour demandé
    deg : booléen (True si on veut la valeur en degrés)
    Returns
    -------
    renvoie la longitude écliptique du Soleil pour la date,
    en radians par défaut, en degrés si deg = True
    """
    Tzero = dt.datetime(2000, 1, 1, 12) #  1er janvier 2000 12hTU, date de référence
    date = dt.datetime(annee, mois, jour)
    T = (date - Tzero).days
    T = T + (date - Tzero).seconds / 3600 / 24
    LM = (280.460 + 0.9856474 * T) % 360
    M = (357.528 + 0.9856003 * T) % 360
    long = (LM + 1.915 * sin(radians(M)) + 0.020 * sin(radians(2*M))) % 360
    if not deg:
        long = radians(long)
    return long




